@@ -10,7 +10,6 @@ require 'cecil-gen-types'
 
 # time to generate code now
 
-$headers = Hash.new
 $types = Hash.new
 $tables = Array.new
 $colls = Array.new
@@ -27,17 +26,6 @@ doc.root.each_element("/cecil/types//type") { |node|
 	$types[type.name] = type
 }
 
-doc.root.each_element("/cecil/headers//header") { |node|
-	header = Cecil::Header.new(node.attribute("name").value)
-	node.each_element("field") { |fi|
-		header.add_field(Cecil::Field.new(
-			fi.attribute("name").value,
-			$types[fi.attribute("type").value],
-			(fi.attribute("default").nil? ? nil : fi.attribute("default").value)))
-	}
-	$headers[header.name] = header
-}
-
 doc.root.each_element("/cecil/metadata/tables//table") { |node|
 	table = Cecil::Table.new(node.attribute("name").value,
 		node.attribute("rid").value,
@@ -143,11 +131,6 @@ def cecil_compile(file, template)
 	end
 end
 
-[ "PEFileHeader.cs", "PEOptionalHeader.cs", "Section.cs",
-  "CLIHeader.cs", "ImageReader.cs", "ImageWriter.cs", "DebugHeader.cs" ].each { |file|
-	cecil_compile(fullpath = "../Mono.Cecil.Binary/" + file, "./templates/" + file)
-}
-
 $tables.each { |table|
 	$cur_table = table
 	filename = "../Mono.Cecil.Metadata/" + table.name + ".cs"
@@ -164,18 +147,15 @@ $cur_table = nil
 cecil_compile("../Mono.Cecil.Metadata/IMetadataVisitor.cs", "./templates/IMetadataVisitor.cs")
 
 cecil_compile("../Mono.Cecil.Cil/OpCodes.cs", "./templates/OpCodes.cs")
+cecil_compile("../Mono.Cecil.Cil/Code.cs", "./templates/Code.cs")
 
 $colls.each { |coll|
 	$cur_coll = coll
-	files = [ "../#{coll.target}/" + coll.intf + ".cs",
-		"../#{coll.target}/" + coll.name + ".cs" ]
+	file = "../#{coll.target}/" + coll.name + ".cs"
 	type = coll.indexed ? "Indexed" : "Named"
-	templates = [ "./templates/I#{type}Collection.cs", "./templates/#{type}CollectionImplem.cs" ]
-	templates[1] = "./templates/Lz#{type}CollectionImplem.cs" if coll.lazyload
-	files.each_with_index { |file, i|
+	template = "./templates/#{type}Collection.cs"
 
-		cecil_compile(file, templates[i])
-	}
+	cecil_compile(file, template)
 }
 $cur_coll = nil
 
