@@ -51,6 +51,8 @@ namespace Mono.Cecil.Binary {
 		DebugHeader m_debugHeader;
 		MetadataRoot m_mdRoot;
 
+		ResourceDirectoryTable m_rsrcRoot;
+
 		FileInfo m_img;
 
 		public DOSHeader DOSHeader {
@@ -103,6 +105,11 @@ namespace Mono.Cecil.Binary {
 			get { return m_hintNameTable; }
 		}
 
+		internal ResourceDirectoryTable ResourceDirectoryRoot {
+			get { return m_rsrcRoot; }
+			set { m_rsrcRoot = value; }
+		}
+
 		public FileInfo FileInformation {
 			get { return m_img; }
 		}
