@@ -4,7 +4,7 @@
 // Author:
 //   Jb Evain (jbevain@gmail.com)
 //
-// (C) 2005 Jb Evain
+// (C) 2005 - 2007 Jb Evain
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -61,7 +61,7 @@ namespace Mono.Cecil.Cil {
 				methBody.CodeSize = flags >> 2;
 				methBody.MaxStack = 8;
 				ReadCilBody (methBody, br, out instrs);
-				return;
+				break;
 			case (int) MethodHeader.FatFormat :
 				br.BaseStream.Position--;
 				int fatflags = br.ReadUInt16 ();
@@ -74,8 +74,11 @@ namespace Mono.Cecil.Cil {
 				ReadCilBody (methBody, br, out instrs);
 				if ((fatflags & (int) MethodHeader.MoreSects) != 0)
 					ReadSection (methBody, br, instrs);
-				return;
+				break;
 			}
+
+			if (m_reflectReader.SymbolReader != null)
+				m_reflectReader.SymbolReader.Read (methBody);
 		}
 
 		public static uint GetRid (int token)
@@ -83,12 +86,7 @@ namespace Mono.Cecil.Cil {
 			return (uint) token & 0x00ffffff;
 		}
 
-		public static bool IsToken (int token, TokenType t)
-		{
-			return token >> 24 == (int) t >> 24;
-		}
-
-		ParameterDefinition GetParameter (MethodBody body, int index)
+		public static ParameterDefinition GetParameter (MethodBody body, int index)
 		{
 			if (body.Method.HasThis) {
 				if (index == 0)
@@ -99,9 +97,14 @@ namespace Mono.Cecil.Cil {
 			return body.Method.Parameters [index];
 		}
 
+		public static VariableDefinition GetVariable (MethodBody body, int index)
+		{
+			return body.Variables [index];
+		}
+
 		void ReadCilBody (MethodBody body, BinaryReader br, out IDictionary instructions)
 		{
-			long start = br.BaseStream.Position, offset;
+			long start = br.BaseStream.Position;
 			Instruction last = null;
 			InstructionCollection code = body.Instructions;
 			instructions = new Hashtable ();
@@ -109,12 +112,12 @@ namespace Mono.Cecil.Cil {
 
 			while (br.BaseStream.Position < start + body.CodeSize) {
 				OpCode op;
-				offset = br.BaseStream.Position - start;
+				long offset = br.BaseStream.Position - start;
 				int cursor = br.ReadByte ();
 				if (cursor == 0xfe)
-					op = OpCodes.Cache.Instance.TwoBytesOpCode [br.ReadByte ()];
+					op = OpCodes.TwoBytesOpCode [br.ReadByte ()];
 				else
-					op = OpCodes.Cache.Instance.OneByteOpCode [cursor];
+					op = OpCodes.OneByteOpCode [cursor];
 
 				Instruction instr = new Instruction ((int) offset, op);
 				switch (op.OperandType) {
@@ -145,7 +148,7 @@ namespace Mono.Cecil.Cil {
 						instr.Operand = br.ReadByte ();
 					break;
 				case OperandType.ShortInlineVar :
-					instr.Operand = body.Variables [br.ReadByte ()];
+					instr.Operand = GetVariable (body, br.ReadByte ());
 					break;
 				case OperandType.ShortInlineParam :
 					instr.Operand = GetParameter (body, br.ReadByte ());
@@ -157,7 +160,7 @@ namespace Mono.Cecil.Cil {
 					instr.Operand = br.ReadInt32 ();
 					break;
 				case OperandType.InlineVar :
-					instr.Operand = body.Variables [br.ReadInt16 ()];
+					instr.Operand = GetVariable (body, br.ReadInt16 ());
 					break;
 				case OperandType.InlineParam :
 					instr.Operand = GetParameter (body, br.ReadInt16 ());
@@ -175,54 +178,77 @@ namespace Mono.Cecil.Cil {
 					instr.Operand = m_root.Streams.UserStringsHeap [GetRid (br.ReadInt32 ())];
 					break;
 				case OperandType.InlineField :
-					int field = br.ReadInt32 ();
-					if (IsToken (field, TokenType.Field))
-						instr.Operand = m_reflectReader.GetFieldDefAt (GetRid (field));
-					else if (IsToken (field, TokenType.MemberRef))
-						instr.Operand = m_reflectReader.GetMemberRefAt (GetRid (field), context);
-					else
-						throw new ReflectionException ("Wrong token for InlineField Operand: {0}", field.ToString ("x8"));
+					MetadataToken field = new MetadataToken (br.ReadInt32 ());
+					switch (field.TokenType) {
+					case TokenType.Field:
+						instr.Operand = m_reflectReader.GetFieldDefAt (field.RID);
+						break;
+					case TokenType.MemberRef:
+						instr.Operand = m_reflectReader.GetMemberRefAt (field.RID, context);
+						break;
+					default:
+						throw new ReflectionException ("Wrong token for InlineField Operand: " + field);
+					}
 					break;
 				case OperandType.InlineMethod :
-					int meth = br.ReadInt32 ();
-					if (IsToken (meth, TokenType.Method))
-						instr.Operand = m_reflectReader.GetMethodDefAt (GetRid (meth));
-					else if (IsToken (meth, TokenType.MemberRef))
-						instr.Operand = m_reflectReader.GetMemberRefAt (GetRid (meth), context);
-					else if (IsToken (meth, TokenType.MethodSpec))
-						instr.Operand = m_reflectReader.GetMethodSpecAt (GetRid (meth), context);
-					else
-						throw new ReflectionException ("Wrong token for InlineMethod Operand: {0}", meth.ToString ("x8"));
+					MetadataToken meth = new MetadataToken (br.ReadInt32 ());
+					switch (meth.TokenType) {
+					case TokenType.Method:
+						instr.Operand = m_reflectReader.GetMethodDefAt (meth.RID);
+						break;
+					case TokenType.MemberRef:
+						instr.Operand = m_reflectReader.GetMemberRefAt (meth.RID, context);
+						break;
+					case TokenType.MethodSpec:
+						instr.Operand = m_reflectReader.GetMethodSpecAt (meth.RID, context);
+						break;
+					default:
+						throw new ReflectionException ("Wrong token for InlineMethod Operand: " + meth);
+					}
 					break;
 				case OperandType.InlineType :
-					int type = br.ReadInt32 ();
-					if (IsToken (type, TokenType.TypeDef))
-						instr.Operand = m_reflectReader.GetTypeDefAt (GetRid (type));
-					else if (IsToken (type, TokenType.TypeRef))
-						instr.Operand = m_reflectReader.GetTypeRefAt (GetRid (type));
-					else if (IsToken (type, TokenType.TypeSpec))
-						instr.Operand = m_reflectReader.GetTypeSpecAt (GetRid (type), context);
-					else
-						throw new ReflectionException ("Wrong token for InlineType Operand: {0}", type.ToString ("x8"));
+					MetadataToken type = new MetadataToken (br.ReadInt32 ());
+					switch (type.TokenType) {
+					case TokenType.TypeDef:
+						instr.Operand = m_reflectReader.GetTypeDefAt (type.RID);
+						break;
+					case TokenType.TypeRef:
+						instr.Operand = m_reflectReader.GetTypeRefAt (type.RID);
+						break;
+					case TokenType.TypeSpec:
+						instr.Operand = m_reflectReader.GetTypeSpecAt (type.RID, context);
+						break;
+					default:
+						throw new ReflectionException ("Wrong token for InlineType Operand: " + type);
+					}
 					break;
 				case OperandType.InlineTok :
-					int token = br.ReadInt32 ();
-					if (IsToken (token, TokenType.TypeDef))
-						instr.Operand = m_reflectReader.GetTypeDefAt (GetRid (token));
-					else if (IsToken (token, TokenType.TypeRef))
-						instr.Operand = m_reflectReader.GetTypeRefAt (GetRid (token));
-					else if (IsToken (token, TokenType.TypeSpec))
-						instr.Operand = m_reflectReader.GetTypeSpecAt (GetRid (token), context);
-					else if (IsToken (token, TokenType.Field))
-						instr.Operand = m_reflectReader.GetFieldDefAt (GetRid (token));
-					else if (IsToken (token, TokenType.Method))
-						instr.Operand = m_reflectReader.GetMethodDefAt (GetRid (token));
-					else if (IsToken (token, TokenType.MethodSpec))
-						instr.Operand = m_reflectReader.GetMethodSpecAt (GetRid (token), context);
-					else if (IsToken (token, TokenType.MemberRef))
-						instr.Operand = m_reflectReader.GetMemberRefAt (GetRid (token), context);
-					else
-						throw new ReflectionException ("Wrong token following ldtoken: {0}", token.ToString ("x8"));
+					MetadataToken token = new MetadataToken (br.ReadInt32 ());
+					switch (token.TokenType) {
+					case TokenType.TypeDef:
+						instr.Operand = m_reflectReader.GetTypeDefAt (token.RID);
+						break;
+					case TokenType.TypeRef:
+						instr.Operand = m_reflectReader.GetTypeRefAt (token.RID);
+						break;
+					case TokenType.TypeSpec:
+						instr.Operand = m_reflectReader.GetTypeSpecAt (token.RID, context);
+						break;
+					case TokenType.Field:
+						instr.Operand = m_reflectReader.GetFieldDefAt (token.RID);
+						break;
+					case TokenType.Method:
+						instr.Operand = m_reflectReader.GetMethodDefAt (token.RID);
+						break;
+					case TokenType.MethodSpec:
+						instr.Operand = m_reflectReader.GetMethodSpecAt (token.RID, context);
+						break;
+					case TokenType.MemberRef:
+						instr.Operand = m_reflectReader.GetMemberRefAt (token.RID, context);
+						break;
+					default:
+						throw new ReflectionException ("Wrong token following ldtoken: " + token);
+					}
 					break;
 				}
 
@@ -243,19 +269,28 @@ namespace Mono.Cecil.Cil {
 				switch (i.OpCode.OperandType) {
 				case OperandType.ShortInlineBrTarget:
 				case OperandType.InlineBrTarget:
-					i.Operand = instructions [(int) i.Operand];
+					i.Operand = GetInstruction (body, instructions, (int) i.Operand);
 					break;
 				case OperandType.InlineSwitch:
 					int [] lbls = (int []) i.Operand;
 					Instruction [] instrs = new Instruction [lbls.Length];
 					for (int j = 0; j < lbls.Length; j++)
-						instrs [j] = instructions [lbls [j]] as Instruction;
+						instrs [j] = GetInstruction (body, instructions, lbls [j]);
 					i.Operand = instrs;
 					break;
 				}
 			}
 		}
 
+		static Instruction GetInstruction (MethodBody body, IDictionary instructions, int offset)
+		{
+			Instruction instruction = instructions [offset] as Instruction;
+			if (instruction != null)
+				return instruction;
+
+			return body.Instructions.Outside;
+		}
+
 		void ReadSection (MethodBody body, BinaryReader br, IDictionary instructions)
 		{
 			br.BaseStream.Position += 3;
@@ -269,26 +304,11 @@ namespace Mono.Cecil.Cil {
 				for (int i = 0; i < length; i++) {
 					ExceptionHandler eh = new ExceptionHandler (
 						(ExceptionHandlerType) (br.ReadInt16 () & 0x7));
-					eh.TryStart = instructions [Convert.ToInt32 (br.ReadInt16 ())] as Instruction;
-					eh.TryEnd = instructions [eh.TryStart.Offset + Convert.ToInt32 (br.ReadByte ())] as Instruction;
-					eh.HandlerStart = instructions [Convert.ToInt32 (br.ReadInt16 ())] as Instruction;
-					eh.HandlerEnd = instructions [eh.HandlerStart.Offset + Convert.ToInt32 (br.ReadByte ())] as Instruction;
-					switch (eh.Type) {
-					case ExceptionHandlerType.Catch :
-						int token = br.ReadInt32 ();
-						if (IsToken (token, TokenType.TypeDef))
-							eh.CatchType = m_reflectReader.GetTypeDefAt (GetRid (token));
-						else
-							eh.CatchType = m_reflectReader.GetTypeRefAt (GetRid (token));
-						break;
-					case ExceptionHandlerType.Filter :
-						eh.FilterStart = instructions [br.ReadInt32 ()] as Instruction;
-						eh.FilterEnd = instructions [eh.HandlerStart.Previous.Offset] as Instruction;
-						break;
-					default :
-						br.ReadInt32 ();
-						break;
-					}
+					eh.TryStart = GetInstruction (body, instructions, Convert.ToInt32 (br.ReadInt16 ()));
+					eh.TryEnd = GetInstruction (body, instructions, eh.TryStart.Offset + Convert.ToInt32 (br.ReadByte ()));
+					eh.HandlerStart = GetInstruction (body, instructions, Convert.ToInt32 (br.ReadInt16 ()));
+					eh.HandlerEnd = GetInstruction (body, instructions, eh.HandlerStart.Offset + Convert.ToInt32 (br.ReadByte ()));
+					ReadExceptionHandlerEnd (eh, br, body, instructions);
 					body.ExceptionHandlers.Add (eh);
 				}
 			} else {
@@ -299,26 +319,11 @@ namespace Mono.Cecil.Cil {
 				for (int i = 0; i < length; i++) {
 					ExceptionHandler eh = new ExceptionHandler (
 						(ExceptionHandlerType) (br.ReadInt32 () & 0x7));
-					eh.TryStart = instructions [br.ReadInt32 ()] as Instruction;
-					eh.TryEnd = instructions [eh.TryStart.Offset + br.ReadInt32 ()] as Instruction;
-					eh.HandlerStart = instructions [br.ReadInt32 ()] as Instruction;
-					eh.HandlerEnd = instructions [eh.HandlerStart.Offset + br.ReadInt32 ()] as Instruction;
-					switch (eh.Type) {
-					case ExceptionHandlerType.Catch :
-						int token = br.ReadInt32 ();
-						if (IsToken (token, TokenType.TypeDef))
-							eh.CatchType = m_reflectReader.GetTypeDefAt (GetRid (token));
-						else
-							eh.CatchType = m_reflectReader.GetTypeRefAt (GetRid (token));
-						break;
-					case ExceptionHandlerType.Filter :
-						eh.FilterStart = instructions [br.ReadInt32 ()] as Instruction;
-						eh.FilterEnd = instructions [eh.HandlerStart.Previous.Offset] as Instruction;
-						break;
-					default :
-						br.ReadInt32 ();
-						break;
-					}
+					eh.TryStart = GetInstruction (body, instructions, br.ReadInt32 ());
+					eh.TryEnd = GetInstruction (body, instructions, eh.TryStart.Offset + br.ReadInt32 ());
+					eh.HandlerStart = GetInstruction (body, instructions, br.ReadInt32 ());
+					eh.HandlerEnd = GetInstruction (body, instructions, eh.HandlerStart.Offset + br.ReadInt32 ());
+					ReadExceptionHandlerEnd(eh, br, body, instructions);
 					body.ExceptionHandlers.Add (eh);
 				}
 			}
@@ -327,6 +332,30 @@ namespace Mono.Cecil.Cil {
 				ReadSection (body, br, instructions);
 		}
 
+		void ReadExceptionHandlerEnd (ExceptionHandler eh, BinaryReader br, MethodBody body, IDictionary instructions)
+		{
+			switch (eh.Type) {
+			case ExceptionHandlerType.Catch :
+				MetadataToken token = new MetadataToken (br.ReadInt32 ());
+				switch (token.TokenType) {
+				case TokenType.TypeDef:
+					eh.CatchType = m_reflectReader.GetTypeDefAt (token.RID);
+					break;
+				default:
+					eh.CatchType = m_reflectReader.GetTypeRefAt (token.RID);
+					break;
+				}
+				break;
+			case ExceptionHandlerType.Filter :
+				eh.FilterStart = GetInstruction (body, instructions, br.ReadInt32 ());
+				eh.FilterEnd = GetInstruction (body, instructions, eh.HandlerStart.Previous.Offset);
+				break;
+			default :
+				br.ReadInt32 ();
+				break;
+			}
+		}
+
 		CallSite GetCallSiteAt (int token, GenericContext context)
 		{
 			StandAloneSigTable sasTable = m_reflectReader.TableReader.GetStandAloneSigTable ();
@@ -334,23 +363,27 @@ namespace Mono.Cecil.Cil {
 				sasTable [(int) GetRid (token) - 1].Signature);
 			CallSite cs = new CallSite (ms.HasThis, ms.ExplicitThis,
 				ms.MethCallConv, m_reflectReader.GetMethodReturnType (ms, context));
-			cs.MetadataToken = new MetadataToken (TokenType.Signature, GetRid (token));
+			cs.MetadataToken = new MetadataToken (token);
 
 			for (int i = 0; i < ms.ParamCount; i++) {
 				Param p = ms.Parameters [i];
 				cs.Parameters.Add (m_reflectReader.BuildParameterDefinition (
 						string.Concat ("A_", i),
-						i, (ParamAttributes) 0,
+						i, (ParameterAttributes) 0,
 						p, context));
 			}
 
+			MethodRefSig refSig = ms as MethodRefSig;
+			if (refSig != null && refSig.Sentinel >= 0)
+				ReflectionReader.CreateSentinel (cs, refSig.Sentinel);
+
 			return cs;
 		}
 
 		public override void VisitVariableDefinitionCollection (VariableDefinitionCollection variables)
 		{
-			MethodBody body = variables.Container;
-			if (body.LocalVarToken == 0)
+			MethodBody body = variables.Container as MethodBody;
+			if (body == null || body.LocalVarToken == 0)
 				return;
 
 			StandAloneSigTable sasTable = m_reflectReader.TableReader.GetStandAloneSigTable ();
