@@ -4,7 +4,7 @@
 // Author:
 //   Jb Evain (jbevain@gmail.com)
 //
-// (C) 2005 Jb Evain
+// (C) 2005 - 2007 Jb Evain
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -43,6 +43,7 @@ namespace Mono.Cecil.Cil {
 		MemoryBinaryWriter m_codeWriter;
 
 		IDictionary m_localSigCache;
+		IDictionary m_standaloneSigCache;
 
 		public CodeWriter (ReflectionWriter reflectWriter, MemoryBinaryWriter writer)
 		{
@@ -51,6 +52,7 @@ namespace Mono.Cecil.Cil {
 			m_codeWriter = new MemoryBinaryWriter ();
 
 			m_localSigCache = new Hashtable ();
+			m_standaloneSigCache = new Hashtable ();
 		}
 
 		public RVA WriteMethodBody (MethodDefinition meth)
@@ -76,7 +78,7 @@ namespace Mono.Cecil.Cil {
 				m_codeWriter.Write (token.ToUInt ());
 		}
 
-		int GetParameterIndex (MethodBody body, ParameterDefinition p)
+		static int GetParameterIndex (MethodBody body, ParameterDefinition p)
 		{
 			int idx = body.Method.Parameters.IndexOf (p);
 			if (idx == -1 && p == body.Method.This)
@@ -113,7 +115,7 @@ namespace Mono.Cecil.Cil {
 				case OperandType.InlineNone :
 					break;
 				case OperandType.InlineSwitch :
-					Instruction [] targets = instr.Operand as Instruction [];
+					Instruction [] targets = (Instruction []) instr.Operand;
 					for (int i = 0; i < targets.Length + 1; i++)
 						m_codeWriter.Write ((uint) 0);
 					break;
@@ -137,7 +139,8 @@ namespace Mono.Cecil.Cil {
 					m_codeWriter.Write ((byte) GetParameterIndex (body, (ParameterDefinition) instr.Operand));
 					break;
 				case OperandType.InlineSig :
-					throw new NotImplementedException ();
+					WriteToken (GetCallSiteToken ((CallSite) instr.Operand));
+					break;
 				case OperandType.InlineI :
 					m_codeWriter.Write ((int) instr.Operand);
 					break;
@@ -198,7 +201,7 @@ namespace Mono.Cecil.Cil {
 					else
 						throw new ReflectionException (
 							string.Format ("Wrong operand for {0} OpCode: {1}",
-								instr.OpCode.OperandType.ToString (),
+								instr.OpCode.OperandType,
 								instr.Operand.GetType ().FullName));
 					break;
 				}
@@ -211,7 +214,7 @@ namespace Mono.Cecil.Cil {
 				switch (instr.OpCode.OperandType) {
 				case OperandType.InlineSwitch :
 					m_codeWriter.BaseStream.Position = instr.Offset + instr.OpCode.Size;
-					Instruction [] targets = instr.Operand as Instruction [];
+					Instruction [] targets = (Instruction []) instr.Operand;
 					m_codeWriter.Write ((uint) targets.Length);
 					foreach (Instruction tgt in targets)
 						m_codeWriter.Write ((tgt.Offset - (instr.Offset +
@@ -219,12 +222,12 @@ namespace Mono.Cecil.Cil {
 					break;
 				case OperandType.ShortInlineBrTarget :
 					m_codeWriter.BaseStream.Position = instr.Offset + instr.OpCode.Size;
-					m_codeWriter.Write ((byte) ((instr.Operand as Instruction).Offset -
+					m_codeWriter.Write ((byte) (((Instruction) instr.Operand).Offset -
 						(instr.Offset + instr.OpCode.Size + 1)));
 					break;
 				case OperandType.InlineBrTarget :
 					m_codeWriter.BaseStream.Position = instr.Offset + instr.OpCode.Size;
-					m_codeWriter.Write ((instr.Operand as Instruction).Offset -
+					m_codeWriter.Write(((Instruction) instr.Operand).Offset -
 						(instr.Offset + instr.OpCode.Size + 4));
 					break;
 				}
@@ -233,33 +236,95 @@ namespace Mono.Cecil.Cil {
 			m_codeWriter.BaseStream.Position = pos;
 		}
 
-		int GetLength (Instruction start, Instruction end, Instruction last)
+		MetadataToken GetCallSiteToken (CallSite cs)
 		{
-			return (end == null ? last.Offset + last.OpCode.Size : end.Offset) - start.Offset;
+			uint sig;
+			int sentinel = cs.GetSentinel ();
+			if (sentinel > 0)
+				sig = m_reflectWriter.SignatureWriter.AddMethodDefSig (
+					m_reflectWriter.GetMethodDefSig (cs));
+			else
+				sig = m_reflectWriter.SignatureWriter.AddMethodRefSig (
+					m_reflectWriter.GetMethodRefSig (cs));
+
+			if (m_standaloneSigCache.Contains (sig))
+				return (MetadataToken) m_standaloneSigCache [sig];
+
+			StandAloneSigTable sasTable = m_reflectWriter.MetadataTableWriter.GetStandAloneSigTable ();
+			StandAloneSigRow sasRow = m_reflectWriter.MetadataRowWriter.CreateStandAloneSigRow (sig);
+
+			sasTable.Rows.Add(sasRow);
+
+			MetadataToken token = new MetadataToken (TokenType.Signature, (uint) sasTable.Rows.Count);
+			m_standaloneSigCache [sig] = token;
+			return token;
 		}
 
-		bool IsRangeFat (Instruction start, Instruction end, Instruction last)
+		static int GetLength (Instruction start, Instruction end, InstructionCollection instructions)
 		{
-			return GetLength (start, end, last) >= 256 ||
+			Instruction last = instructions [instructions.Count - 1];
+			return (end == instructions.Outside ? last.Offset + GetSize (last) : end.Offset) - start.Offset;
+		}
+
+		static int GetSize (Instruction i)
+		{
+			int size = i.OpCode.Size;
+
+			switch (i.OpCode.OperandType) {
+			case OperandType.InlineSwitch:
+				size += ((Instruction []) i.Operand).Length * 4;
+				break;
+			case OperandType.InlineI8:
+			case OperandType.InlineR:
+				size += 8;
+				break;
+			case OperandType.InlineBrTarget:
+			case OperandType.InlineField:
+			case OperandType.InlineI:
+			case OperandType.InlineMethod:
+			case OperandType.InlineString:
+			case OperandType.InlineTok:
+			case OperandType.InlineType:
+			case OperandType.ShortInlineR:
+				size += 4;
+				break;
+			case OperandType.InlineParam:
+			case OperandType.InlineVar:
+				size += 2;
+				break;
+			case OperandType.ShortInlineBrTarget:
+			case OperandType.ShortInlineI:
+			case OperandType.ShortInlineParam:
+			case OperandType.ShortInlineVar:
+				size += 1;
+				break;
+			}
+
+			return size;
+		}
+
+		static bool IsRangeFat (Instruction start, Instruction end, InstructionCollection instructions)
+		{
+			return GetLength (start, end, instructions) >= 256 ||
 				start.Offset >= 65536;
 		}
 
-		bool IsFat (ExceptionHandlerCollection seh, Instruction last)
+		static bool IsFat (ExceptionHandlerCollection seh)
 		{
 			for (int i = 0; i < seh.Count; i++) {
 				ExceptionHandler eh = seh [i];
-				if (IsRangeFat (eh.TryStart, eh.TryEnd, last))
+				if (IsRangeFat (eh.TryStart, eh.TryEnd, seh.Container.Instructions))
 					return true;
 
 				switch (eh.Type) {
 				case ExceptionHandlerType.Catch :
 				case ExceptionHandlerType.Fault :
 				case ExceptionHandlerType.Finally :
-					if (IsRangeFat (eh.HandlerStart, eh.HandlerEnd, last))
+					if (IsRangeFat (eh.HandlerStart, eh.HandlerEnd, seh.Container.Instructions))
 						return true;
 					break;
 				case ExceptionHandlerType.Filter :
-					if (IsRangeFat (eh.FilterStart, eh.FilterEnd, last))
+					if (IsRangeFat (eh.FilterStart, eh.FilterEnd, seh.Container.Instructions))
 						return true;
 					break;
 				}
@@ -271,9 +336,8 @@ namespace Mono.Cecil.Cil {
 		void WriteExceptionHandlerCollection (ExceptionHandlerCollection seh)
 		{
 			m_codeWriter.QuadAlign ();
-			Instruction last = seh.Container.Instructions [seh.Container.Instructions.Count - 1];
 
-			if (!IsFat (seh, last)) {
+			if (seh.Count < 0x15 && !IsFat (seh)) {
 				m_codeWriter.Write ((byte) MethodDataSection.EHTable);
 				m_codeWriter.Write ((byte) (seh.Count * 12 + 4));
 				m_codeWriter.Write (new byte [2]);
@@ -282,7 +346,7 @@ namespace Mono.Cecil.Cil {
 					m_codeWriter.Write ((ushort) eh.TryStart.Offset);
 					m_codeWriter.Write ((byte) (eh.TryEnd.Offset - eh.TryStart.Offset));
 					m_codeWriter.Write ((ushort) eh.HandlerStart.Offset);
-					m_codeWriter.Write ((byte) GetLength (eh.HandlerStart, eh.HandlerEnd, last));
+					m_codeWriter.Write ((byte) GetLength (eh.HandlerStart, eh.HandlerEnd, seh.Container.Instructions));
 					WriteHandlerSpecific (eh);
 				}
 			} else {
@@ -293,7 +357,7 @@ namespace Mono.Cecil.Cil {
 					m_codeWriter.Write ((uint) eh.TryStart.Offset);
 					m_codeWriter.Write ((uint) (eh.TryEnd.Offset - eh.TryStart.Offset));
 					m_codeWriter.Write ((uint) eh.HandlerStart.Offset);
-					m_codeWriter.Write ((uint) GetLength (eh.HandlerStart, eh.HandlerEnd, last));
+					m_codeWriter.Write ((uint) GetLength (eh.HandlerStart, eh.HandlerEnd, seh.Container.Instructions));
 					WriteHandlerSpecific (eh);
 				}
 			}
@@ -324,7 +388,10 @@ namespace Mono.Cecil.Cil {
 
 		public override void VisitVariableDefinitionCollection (VariableDefinitionCollection variables)
 		{
-			MethodBody body = variables.Container;
+			MethodBody body = variables.Container as MethodBody;
+			if (body == null)
+				return;
+
 			uint sig = m_reflectWriter.SignatureWriter.AddLocalVarSig (
 					GetLocalVarSig (variables));
 
@@ -402,7 +469,7 @@ namespace Mono.Cecil.Cil {
 			return lvs;
 		}
 
-		void ComputeMaxStack (InstructionCollection instructions)
+		static void ComputeMaxStack (InstructionCollection instructions)
 		{
 			InstructionCollection ehs = new InstructionCollection (null);
 			foreach (ExceptionHandler eh in instructions.Container.ExceptionHandlers)
@@ -466,6 +533,9 @@ namespace Mono.Cecil.Cil {
 					current -= 3;
 					break;
 				}
+
+				if (current < 0)
+					current = 0;
 			}
 
 			instructions.Container.MaxStack = max;
