@@ -4,7 +4,7 @@
 // Author:
 //   Jb Evain (jbevain@gmail.com)
 //
-// (C) 2005 Jb Evain
+// (C) 2005 - 2007 Jb Evain
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -152,6 +152,7 @@ namespace Mono.Cecil.Signatures {
 		{
 			m_sigWriter.Write (property.CallingConvention);
 			Write (property.ParamCount);
+			Write (property.CustomMods);
 			Write (property.Type);
 			Write (property.Parameters);
 		}
@@ -235,6 +236,7 @@ namespace Mono.Cecil.Signatures {
 				break;
 			case ElementType.Array :
 				ARRAY ary = (ARRAY) t;
+				Write (ary.CustomMods);
 				ArrayShape shape = ary.Shape;
 				Write (ary.Type);
 				Write (shape.Rank);
@@ -246,7 +248,9 @@ namespace Mono.Cecil.Signatures {
 					Write (loBound);
 				break;
 			case ElementType.SzArray :
-				Write (((SZARRAY) t).Type);
+				SZARRAY sa = (SZARRAY) t;
+				Write (sa.CustomMods);
+				Write (sa.Type);
 				break;
 			case ElementType.Var :
 				Write (((VAR) t).Index);
@@ -282,6 +286,12 @@ namespace Mono.Cecil.Signatures {
 				Write (gis.Types [i]);
 		}
 
+		void Write (GenericArg arg)
+		{
+			Write (arg.CustomMods);
+			Write (arg.Type);
+		}
+
 		void Write (Param p)
 		{
 			Write (p.CustomMods);
