@@ -150,6 +150,9 @@ namespace Mono.Cecil {
 					prow.Flags);
 					pdef.MetadataToken = MetadataToken.FromMetadataRow (TokenType.Property, j - 1);
 
+					if (psig.CustomMods != null && psig.CustomMods.Length > 0)
+						pdef.PropertyType = GetModifierType (psig.CustomMods, pdef.PropertyType);
+
 					owner.Properties.Add (pdef);
 					m_properties [j - 1] = pdef;
 				}
@@ -302,7 +305,7 @@ namespace Mono.Cecil {
 				CustomAttribute cattr;
 				if (!ca.Read) {
 					cattr = new CustomAttribute (ctor);
-					cattr.IsReadable = false;
+					cattr.Resolved = false;
 					cattr.Blob = m_root.Streams.BlobHeap.Read (caRow.Value);
 				} else
 					cattr = BuildCustomAttribute (ctor, ca);
