@@ -31,7 +31,7 @@ namespace Mono.Cecil {
 	using System;
 	using System.Collections;
 
-	public sealed class CustomAttribute : ICustomAttribute, ICloneable {
+	public sealed class CustomAttribute : IRequireResolving, IReflectionVisitable {
 
 		MethodReference m_ctor;
 		IList m_parameters;
@@ -40,7 +40,7 @@ namespace Mono.Cecil {
 		IDictionary m_fieldTypes;
 		IDictionary m_propTypes;
 
-		bool m_readable;
+		bool m_resolved;
 		byte [] m_blob;
 
 		public MethodReference Constructor {
@@ -92,9 +92,9 @@ namespace Mono.Cecil {
 			}
 		}
 
-		public bool IsReadable {
-			get { return m_readable; }
-			set { m_readable = value; }
+		public bool Resolved {
+			get { return m_resolved; }
+			set { m_resolved = value; }
 		}
 
 		public byte [] Blob {
@@ -105,32 +105,27 @@ namespace Mono.Cecil {
 		public CustomAttribute (MethodReference ctor)
 		{
 			m_ctor = ctor;
-			m_readable = true;
+			m_resolved = true;
 		}
 
 		public TypeReference GetFieldType (string fieldName)
 		{
-			return (TypeReference) this.FieldTypes [fieldName];
+			return (TypeReference) FieldTypes [fieldName];
 		}
 
 		public TypeReference GetPropertyType (string propertyName)
 		{
-			return (TypeReference) this.PropertyTypes [propertyName];
+			return (TypeReference) PropertyTypes [propertyName];
 		}
 
-		public void SetFieldType (string fieldName, ITypeReference type)
+		public void SetFieldType (string fieldName, TypeReference type)
 		{
-			this.FieldTypes [fieldName] = type;
+			FieldTypes [fieldName] = type;
 		}
 
-		public void SetPropertyType (string propertyName, ITypeReference type)
+		public void SetPropertyType (string propertyName, TypeReference type)
 		{
-			this.PropertyTypes [propertyName] = type;
-		}
-
-		object ICloneable.Clone ()
-		{
-			return this.Clone ();
+			PropertyTypes [propertyName] = type;
 		}
 
 		public CustomAttribute Clone ()
@@ -140,6 +135,7 @@ namespace Mono.Cecil {
 
 		static void Clone (IDictionary original, IDictionary target)
 		{
+			target.Clear ();
 			foreach (DictionaryEntry entry in original)
 				target.Add (entry.Key, entry.Value);
 		}
@@ -147,19 +143,38 @@ namespace Mono.Cecil {
 		internal static CustomAttribute Clone (CustomAttribute custattr, ImportContext context)
 		{
 			CustomAttribute ca = new CustomAttribute (context.Import (custattr.Constructor));
-			if (!custattr.IsReadable) {
-				ca.IsReadable = false;
-				ca.Blob = custattr.Blob;
-				return ca;
+			custattr.CopyTo (ca);
+			return ca;
+		}
+
+		void CopyTo (CustomAttribute target)
+		{
+			target.Resolved = Resolved;
+			if (!Resolved) {
+				target.Blob = Blob;
+				return;
 			}
 
-			foreach (object o in custattr.ConstructorParameters)
-				ca.ConstructorParameters.Add (o);
-			Clone (custattr.Fields, ca.Fields);
-			Clone (custattr.FieldTypes, ca.FieldTypes);
-			Clone (custattr.Properties, ca.Properties);
-			Clone (custattr.PropertyTypes, ca.PropertyTypes);
-			return ca;
+			foreach (object o in ConstructorParameters)
+				target.ConstructorParameters.Add (o);
+			Clone (Fields, target.Fields);
+			Clone (FieldTypes, target.FieldTypes);
+			Clone (Properties, target.Properties);
+			Clone (PropertyTypes, target.PropertyTypes);
+		}
+
+		public bool Resolve ()
+		{
+			if (Resolved)
+				return true;
+
+			ReflectionReader r = m_ctor.DeclaringType.Module.Controller.Reader;
+			CustomAttribute newCa = r.GetCustomAttribute (m_ctor, Blob, true);
+			if (!newCa.Resolved)
+				return false;
+
+			newCa.CopyTo (this);
+			return true;
 		}
 
 		public void Accept (IReflectionVisitor visitor)
