@@ -4,7 +4,7 @@
 // Author:
 //   Jb Evain (jbevain@gmail.com)
 //
-// (C) 2005 Jb Evain
+// (C) 2005 - 2007 Jb Evain
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -28,17 +28,10 @@
 
 namespace Mono.Cecil {
 
-	using System;
+	public sealed class ParameterDefinition : ParameterReference, IHasMarshalSpec,
+		IMetadataTokenProvider, ICustomAttributeProvider, IHasConstant {
 
-	using Mono.Cecil.Metadata;
-
-	public sealed class ParameterDefinition : IParameterDefinition, ICloneable {
-
-		string m_name;
-		int m_sequence;
-		ParamAttributes m_attributes;
-		TypeReference m_paramType;
-		MetadataToken m_token;
+		ParameterAttributes m_attributes;
 
 		bool m_hasConstant;
 		object m_const;
@@ -46,33 +39,13 @@ namespace Mono.Cecil {
 		MethodReference m_method;
 		CustomAttributeCollection m_customAttrs;
 
-		MarshalDesc m_marshalDesc;
+		MarshalSpec m_marshalDesc;
 
-		public string Name {
-			get { return m_name; }
-			set { m_name = value; }
-		}
-
-		public int Sequence {
-			get { return m_sequence; }
-			set { m_sequence = value; }
-		}
-
-		public ParamAttributes Attributes {
+		public ParameterAttributes Attributes {
 			get { return m_attributes; }
 			set { m_attributes = value; }
 		}
 
-		public TypeReference ParameterType {
-			get { return m_paramType; }
-			set { m_paramType = value; }
-		}
-
-		public MetadataToken MetadataToken {
-			get { return m_token; }
-			set { m_token = value; }
-		}
-
 		public bool HasConstant {
 			get { return m_hasConstant; }
 		}
@@ -99,27 +72,19 @@ namespace Mono.Cecil {
 			}
 		}
 
-		public MarshalDesc MarshalSpec {
+		public MarshalSpec MarshalSpec {
 			get { return m_marshalDesc; }
 			set { m_marshalDesc = value; }
 		}
 
 		public ParameterDefinition (TypeReference paramType) :
-			this (string.Empty, -1, (ParamAttributes) 0, paramType)
+			this (string.Empty, -1, (ParameterAttributes) 0, paramType)
 		{
 		}
 
-		public ParameterDefinition (string name, int seq, ParamAttributes attrs, TypeReference paramType)
+		public ParameterDefinition (string name, int seq, ParameterAttributes attrs, TypeReference paramType) : base (name, seq, paramType)
 		{
-			m_name = name;
-			m_sequence = seq;
 			m_attributes = attrs;
-			m_paramType = paramType;
-		}
-
-		object ICloneable.Clone ()
-		{
-			return this.Clone ();
 		}
 
 		public ParameterDefinition Clone ()
@@ -147,15 +112,7 @@ namespace Mono.Cecil {
 			return np;
 		}
 
-		public override string ToString ()
-		{
-			if (m_name != null && m_name.Length > 0)
-				return m_name;
-
-			return string.Concat ("A_", m_sequence);
-		}
-
-		public void Accept (IReflectionVisitor visitor)
+		public override void Accept (IReflectionVisitor visitor)
 		{
 			visitor.VisitParameterDefinition (this);
 
