@@ -6,6 +6,7 @@
 //
 // (C) 2005 Jb Evain
 // (C) 2006 Evaluant RC S.A.
+// (C) 2007 Jb Evain
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -79,7 +80,6 @@ namespace Mono.Cecil {
 			asmRef.PublicKeyToken = asmName.GetPublicKeyToken ();
 			asmRef.HashAlgorithm = (Mono.Cecil.AssemblyHashAlgorithm) asmName.HashAlgorithm;
 			asmRef.Culture = asmName.CultureInfo.ToString ();
-			asmRef.Flags = (Mono.Cecil.AssemblyFlags) asmName.Flags;
 			m_module.AssemblyReferences.Add (asmRef);
 			m_asmCache [asm.FullName] = asmRef;
 			return asmRef;
@@ -415,7 +415,6 @@ namespace Mono.Cecil {
 				asm.Name, asm.Culture, asm.Version);
 			asmRef.PublicKeyToken = asm.PublicKeyToken;
 			asmRef.HashAlgorithm = asm.HashAlgorithm;
-			asmRef.Flags = asm.Flags;
 			m_module.AssemblyReferences.Add (asmRef);
 			m_asmCache [asm.FullName] = asmRef;
 			return asmRef;
@@ -445,14 +444,14 @@ namespace Mono.Cecil {
 						genElemType.GenericArguments.Add (ImportTypeReference (arg, context));
 
 					elementType = genElemType;
-				}else
+				} else
 					throw new ReflectionException ("Unknown element type: {0}", t.GetType ().Name);
 			}
 
 			return elementType;
 		}
 
-		GenericParameter GetGenericParameter (GenericParameter gp, ImportContext context)
+		static GenericParameter GetGenericParameter (GenericParameter gp, ImportContext context)
 		{
 			GenericParameter p;
 			if (gp.Owner is TypeReference)
