@@ -4,7 +4,7 @@
 // Author:
 //   Jb Evain (jbevain@gmail.com)
 //
-// (C) 2005 Jb Evain
+// (C) 2005 - 2007 Jb Evain
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -29,10 +29,7 @@
 namespace Mono.Cecil {
 
 	using System;
-	using System.Collections;
 	using System.IO;
-	using System.Security;
-	using System.Security.Permissions;
 	using System.Text;
 
 	using Mono.Cecil.Binary;
@@ -66,6 +63,23 @@ namespace Mono.Cecil {
 
 		protected SignatureReader m_sigReader;
 		protected CodeReader m_codeReader;
+		protected ISymbolReader m_symbolReader;
+
+		internal AssemblyNameReference Corlib {
+			get {
+				if (m_corlib != null)
+					return m_corlib;
+
+				foreach (AssemblyNameReference ar in m_module.AssemblyReferences) {
+					if (ar.Name == Constants.Corlib) {
+						m_corlib = ar;
+						return m_corlib;
+					}
+				}
+
+				return null;
+			}
+		}
 
 		public ModuleDefinition Module {
 			get { return m_module; }
@@ -83,6 +97,11 @@ namespace Mono.Cecil {
 			get { return m_codeReader; }
 		}
 
+		public ISymbolReader SymbolReader {
+			get { return m_symbolReader; }
+			set { m_symbolReader = value; }
+		}
+
 		public MetadataRoot MetadataRoot {
 			get { return m_root; }
 		}
@@ -172,7 +191,7 @@ namespace Mono.Cecil {
 						GetTypeRefFromSig (fs.Type, nc));
 				} else {
 					string name = m_root.Streams.StringsHeap [mrefRow.Name];
-					MethodSig ms = sig as MethodSig;
+					MethodSig ms = (MethodSig) sig;
 
 					MethodReference methref = new MethodReference (
 						name, ms.HasThis, ms.ExplicitThis, ms.MethCallConv);
@@ -184,7 +203,8 @@ namespace Mono.Cecil {
 							methref.GenericParameters.Add (new GenericParameter (i, methref));
 					}
 
-					nc.Method = methref;
+					if (methref.GenericParameters.Count > 0)
+						nc.Method = methref;
 
 					methref.ReturnType = GetMethodReturnType (ms, nc);
 
@@ -192,21 +212,25 @@ namespace Mono.Cecil {
 					for (int j = 0; j < ms.ParamCount; j++) {
 						Param p = ms.Parameters [j];
 						ParameterDefinition pdef = BuildParameterDefinition (
-							string.Concat ("A_", j), j, new ParamAttributes (), p, nc);
+							string.Concat ("A_", j), j, new ParameterAttributes (), p, nc);
 						pdef.Method = methref;
 						methref.Parameters.Add (pdef);
 					}
 					member = methref;
-					nc.Method = methref;
 				}
 				break;
 			case TokenType.Method :
 				// really not sure about this
 				MethodDefinition methdef = GetMethodDefAt (mrefRow.Class.RID);
-				member = new MethodReference (
+				MethodReference methRef = new MethodReference (
 					methdef.Name, methdef.HasThis,
 					methdef.ExplicitThis, methdef.CallingConvention);
-				member.DeclaringType = methdef.DeclaringType;
+
+				methRef.DeclaringType = methdef.DeclaringType;
+				methRef.ReturnType = methdef.ReturnType;
+				foreach (ParameterDefinition param in methdef.Parameters)
+					methRef.Parameters.Add (param);
+				member = methRef;
 				break;
 			case TokenType.ModuleRef :
 				break; // TODO, implement that, or not
@@ -260,8 +284,8 @@ namespace Mono.Cecil {
 				throw new ReflectionException ("Unknown method type for method spec");
 
 			gim = new GenericInstanceMethod (meth);
-			foreach (SigType st in sig.Signature.Types)
-				gim.GenericArguments.Add (GetTypeRefFromSig (st, context));
+			foreach (GenericArg arg in sig.Signature.Types)
+				gim.GenericArguments.Add (GetGenericArg (arg, context));
 
 			m_methodSpecs [index] = gim;
 
@@ -292,19 +316,11 @@ namespace Mono.Cecil {
 
 			TypeReference coreType =  m_module.TypeReferences [fullName];
 			if (coreType == null) {
-				if (m_corlib == null) {
-					foreach (AssemblyNameReference ar in m_module.AssemblyReferences) {
-						if (ar.Name == Constants.Corlib) {
-							m_corlib = ar;
-							break;
-						}
-					}
-				}
 
 				string [] parts = fullName.Split ('.');
 				if (parts.Length != 2)
 					throw new ReflectionException ("Unvalid core type name");
-				coreType = new TypeReference (parts [1], parts [0], m_corlib);
+				coreType = new TypeReference (parts [1], parts [0], Corlib);
 				m_module.TypeReferences.Add (coreType);
 			}
 			if (!coreType.IsValueType) {
@@ -352,10 +368,15 @@ namespace Mono.Cecil {
 			}
 		}
 
+		public CustomAttribute GetCustomAttribute (MethodReference ctor, byte [] data, bool resolve)
+		{
+			CustomAttrib sig = m_sigReader.GetCustomAttrib (data, ctor, resolve);
+			return BuildCustomAttribute (ctor, sig);
+		}
+
 		public CustomAttribute GetCustomAttribute (MethodReference ctor, byte [] data)
 		{
-			CustomAttrib sig = m_sigReader.GetCustomAttrib (data, ctor);
-			return BuildCustomAttribute (ctor, sig);
+			return GetCustomAttribute (ctor, data, false);
 		}
 
 		public override void VisitModuleDefinition (ModuleDefinition mod)
@@ -539,7 +560,7 @@ namespace Mono.Cecil {
 					FieldSig fsig = m_sigReader.GetFieldSig (frow.Signature);
 					FieldDefinition fdef = new FieldDefinition (
 						m_root.Streams.StringsHeap [frow.Name],
-						this.GetTypeRefFromSig (fsig.Type, context), frow.Flags);
+						GetTypeRefFromSig (fsig.Type, context), frow.Flags);
 					fdef.MetadataToken = MetadataToken.FromMetadataRow (TokenType.Field, j - 1);
 
 					if (fsig.CustomMods.Length > 0)
@@ -623,27 +644,32 @@ namespace Mono.Cecil {
 					ParameterDefinition retparam = null;
 
 					//TODO: optimize this
-					ParamRow pRow = null;
 					int start = (int) methRow.ParamList - 1;
 
-					if (paramTable != null && start < prms - 1)
-						pRow = paramTable [start];
+					if (paramTable != null && start < prms - 1) {
 
-					if (pRow != null && pRow.Sequence == 0) { // ret type
-						retparam = new ParameterDefinition (
-							m_root.Streams.StringsHeap [pRow.Name],
-							0,
-							pRow.Flags,
-							null);
-						retparam.Method = mdef;
-						m_parameters [start] = retparam;
-						start++;
+						ParamRow pRetRow = paramTable [start];
+
+						if (pRetRow != null && pRetRow.Sequence == 0) { // ret type
+
+							retparam = new ParameterDefinition (
+								m_root.Streams.StringsHeap [pRetRow.Name],
+								0,
+								pRetRow.Flags,
+								null);
+
+							retparam.Method = mdef;
+							m_parameters [start] = retparam;
+							start++;
+						}
 					}
 
 					for (int k = 0; k < msig.ParamCount; k++) {
 
 						int pointer = start + k;
 
+						ParamRow pRow = null;
+
 						if (paramTable != null && pointer < prms - 1)
 							pRow = paramTable [pointer];
 
@@ -659,7 +685,7 @@ namespace Mono.Cecil {
 						} else
 							pdef = BuildParameterDefinition (
 								string.Concat ("A_", mdef.IsStatic ? k : k + 1),
-								k + 1, (ParamAttributes) 0, psig, context);
+								k + 1, (ParameterAttributes) 0, psig, context);
 
 						pdef.Method = mdef;
 						mdef.Parameters.Add (pdef);
@@ -729,7 +755,7 @@ namespace Mono.Cecil {
 			}
 		}
 
-		object GetFixedArgValue (CustomAttrib.FixedArg fa)
+		static object GetFixedArgValue (CustomAttrib.FixedArg fa)
 		{
 			if (fa.SzArray) {
 				object [] vals = new object [fa.NumElem];
@@ -774,7 +800,7 @@ namespace Mono.Cecil {
 		}
 
 		public ParameterDefinition BuildParameterDefinition (string name, int sequence,
-			ParamAttributes attrs, Param psig, GenericContext context)
+			ParameterAttributes attrs, Param psig, GenericContext context)
 		{
 			ParameterDefinition ret = new ParameterDefinition (name, sequence, attrs, null);
 			TypeReference paramType;
@@ -799,17 +825,18 @@ namespace Mono.Cecil {
 			return BuildSecurityDeclaration (dsRow.Action, m_root.Streams.BlobHeap.Read (dsRow.PermissionSet));
 		}
 
-		public SecurityDeclaration BuildSecurityDeclaration (Mono.Cecil.SecurityAction action, byte [] permset)
+		public SecurityDeclaration BuildSecurityDeclaration (SecurityAction action, byte [] permset)
 		{
 			if (m_secReader == null)
 				m_secReader = new SecurityDeclarationReader (m_root, this);
+
 			return m_secReader.FromByteArray (action, permset);
 		}
 
-		protected MarshalDesc BuildMarshalDesc (MarshalSig ms, IHasMarshalSpec container)
+		protected MarshalSpec BuildMarshalDesc (MarshalSig ms, IHasMarshalSpec container)
 		{
 			if (ms.Spec is MarshalSig.Array) {
-				ArrayMarshalDesc amd = new ArrayMarshalDesc (container);
+				ArrayMarshalSpec amd = new ArrayMarshalSpec (container);
 				MarshalSig.Array ar = (MarshalSig.Array) ms.Spec;
 				amd.ElemType = ar.ArrayElemType;
 				amd.NumElem = ar.NumElem;
@@ -817,7 +844,7 @@ namespace Mono.Cecil {
 				amd.ElemMult = ar.ElemMult;
 				return amd;
 			} else if (ms.Spec is MarshalSig.CustomMarshaler) {
-				CustomMarshalerDesc cmd = new CustomMarshalerDesc (container);
+				CustomMarshalerSpec cmd = new CustomMarshalerSpec (container);
 				MarshalSig.CustomMarshaler cmsig = (MarshalSig.CustomMarshaler) ms.Spec;
 				cmd.Guid = cmsig.Guid.Length > 0 ? new Guid (cmsig.Guid) : new Guid ();
 				cmd.UnmanagedType = cmsig.UnmanagedType;
@@ -825,21 +852,21 @@ namespace Mono.Cecil {
 				cmd.Cookie = cmsig.Cookie;
 				return cmd;
 			} else if (ms.Spec is MarshalSig.FixedArray) {
-				FixedArrayDesc fad = new FixedArrayDesc (container);
+				FixedArraySpec fad = new FixedArraySpec (container);
 				MarshalSig.FixedArray fasig = (MarshalSig.FixedArray) ms.Spec;
 				fad.ElemType = fasig.ArrayElemType;
 				fad.NumElem = fasig.NumElem;
 				return fad;
 			} else if (ms.Spec is MarshalSig.FixedSysString) {
-				FixedSysStringDesc fssc = new FixedSysStringDesc (container);
+				FixedSysStringSpec fssc = new FixedSysStringSpec (container);
 				fssc.Size = ((MarshalSig.FixedSysString) ms.Spec).Size;
 				return fssc;
 			} else if (ms.Spec is MarshalSig.SafeArray) {
-				SafeArrayDesc sad = new SafeArrayDesc (container);
+				SafeArraySpec sad = new SafeArraySpec (container);
 				sad.ElemType = ((MarshalSig.SafeArray) ms.Spec).ArrayElemType;
 				return sad;
 			} else {
-				return new MarshalDesc (ms.NativeInstrinsic, container);
+				return new MarshalSpec (ms.NativeInstrinsic, container);
 			}
 		}
 
@@ -949,9 +976,14 @@ namespace Mono.Cecil {
 					Param p = funcptr.Method.Parameters [i];
 					fnptr.Parameters.Add (BuildParameterDefinition (
 							string.Concat ("A_", i),
-							i, (ParamAttributes) 0,
+							i, (ParameterAttributes) 0,
 							p, context));
 				}
+
+				MethodRefSig refSig = funcptr.Method as MethodRefSig;
+				if (refSig != null && refSig.Sentinel >= 0)
+					CreateSentinel (fnptr, refSig.Sentinel);
+
 				return fnptr;
 			case ElementType.Var:
 				VAR var = t as VAR;
@@ -974,7 +1006,7 @@ namespace Mono.Cecil {
 				instance.IsValueType = ginst.ValueType;
 
 				for (int i = 0; i < ginst.Signature.Arity; i++)
-					instance.GenericArguments.Add (GetTypeRefFromSig (
+					instance.GenericArguments.Add (GetGenericArg (
 						ginst.Signature.Types [i], context));
 
 				return instance;
@@ -984,7 +1016,24 @@ namespace Mono.Cecil {
 			return null;
 		}
 
-		void CheckGenericParameters (GenericContext context, VAR v)
+		public static void CreateSentinel (IMethodSignature meth, int sentinel)
+		{
+			if (sentinel < 0 || sentinel >= meth.Parameters.Count)
+				throw new ArgumentException ("Invalid sentinel");
+
+			ParameterDefinition param = meth.Parameters [sentinel];
+			param.ParameterType = new SentinelType (param.ParameterType);
+		}
+
+		TypeReference GetGenericArg (GenericArg arg, GenericContext context)
+		{
+			TypeReference type = GetTypeRefFromSig (arg.Type, context);
+			if (arg.CustomMods != null && arg.CustomMods.Length > 0)
+				type = GetModifierType (arg.CustomMods, type);
+			return type;
+		}
+
+		static void CheckGenericParameters (GenericContext context, VAR v)
 		{
 			for (int i = context.Type.GenericParameters.Count; i <= v.Index; i++)
 				context.Type.GenericParameters.Add (new GenericParameter (i, context.Type));
@@ -1034,7 +1083,8 @@ namespace Mono.Cecil {
 		protected void SetInitialValue (FieldDefinition field)
 		{
 			int size = 0;
-			switch (field.FieldType.FullName) {
+			TypeReference fieldType = field.FieldType;
+			switch (fieldType.FullName) {
 			case Constants.Byte:
 			case Constants.SByte:
 				size = 1;
@@ -1055,9 +1105,13 @@ namespace Mono.Cecil {
 				size = 8;
 				break;
 			default:
-				TypeDefinition fieldType = field.FieldType as TypeDefinition;
-				if (fieldType != null)
-					size = (int) fieldType.ClassSize;
+				while (fieldType is TypeSpecification)
+					fieldType = ((TypeSpecification) fieldType).ElementType;
+
+				TypeDefinition fieldTypeDef = fieldType as TypeDefinition;
+
+				if (fieldTypeDef != null)
+					size = (int) fieldTypeDef.ClassSize;
 				break;
 			}
 
